/* eslint-env mocha */
'use strict'

const chai = require('chai')
const dirtyChai = require('dirty-chai')
const expect = chai.expect
chai.use(dirtyChai)

const loadFixture = require('aegir/fixtures')
const ipfs = require('ipfs')
const DaemonFactory = require('ipfsd-ctl')
const CID = require('cids')


const df = DaemonFactory.create({ type: 'proc', exec: ipfs })
const cidv1b32 = (cid) => new CID(cid).toBaseEncodedString('base32')

describe('ipfs.files.add then ipfs.files.cat with CIDv0', function () {
  let ipfs = null
  let ipfsd = null

  const file = {
    cid: 'Qma4hjFTnCasJ8PVp3mZbZK5g2vGDT4LByLJ7m8ciyRFZP',
    data: loadFixture('test/fixtures/testfile.txt')
  }

  before(function (done) {
    this.timeout(20 * 1000)

    df.spawn({ initOptions: { bits: 512 } }, (err, _ipfsd) => {
      expect(err).to.not.exist()
      ipfsd = _ipfsd
      ipfs = ipfsd.api

      ipfs.files.add(file.data, {cidVersion: 0}, (err, filesAdded) => {
        expect(err).to.not.exist()
        expect(filesAdded).to.have.length(1)
        console.log('Result of ipfs.files.add (CIDv0):', filesAdded)

        const addedFile = filesAdded[0]
        expect(addedFile.size, 'ipfs.files.add result.size should not be smaller than input buffer').greaterThan(file.data.length)
        expect(new CID(addedFile.hash)).to.deep.equal(new CID(file.cid))

        done()
      })
    })
  })

  it('ipfs.files.cat returns data back (CIDv0)', async () => {
    const data = await ipfs.files.cat(`/ipfs/${file.cid}`)
    expect(data).to.exist()
    const expectedContents = loadFixture('test/fixtures/testfile.txt')
    expect(data.toString()).to.equal(expectedContents.toString())
  })
})

describe('ipfs.files.add then ipfs.files.cat with CIDv1', function () {
  let ipfs = null
  let ipfsd = null

  const file = {
    // cid below produced by go-ipfs 0.4.17: ipfs add --cid-version 1 test/fixtures/testfile.txt
    cid: 'bafkreidffqfydlguosmmyebv5rp72m45tbpbq6segnkosa45kjfnduix6u',
    data: loadFixture('test/fixtures/testfile.txt')
  }

  before(function (done) {
    this.timeout(20 * 1000)

    df.spawn({ initOptions: { bits: 512 } }, (err, _ipfsd) => {
      expect(err).to.not.exist()
      ipfsd = _ipfsd
      ipfs = ipfsd.api

      ipfs.files.add(file.data, {cidVersion: 1}, (err, filesAdded) => {
        expect(err).to.not.exist()
        expect(filesAdded).to.have.length(1)
        console.log('Result of ipfs.files.add (CIDv1):', filesAdded)

        const addedFile = filesAdded[0]
        expect(addedFile.size, 'ipfs.files.add result.size should not be smaller than input buffer').greaterThan(file.data.length)
        expect(new CID(addedFile.hash)).to.deep.equal(new CID(file.cid))

        done()
      })
    })
  })

  it('ipfs.files.cat returns data back (CIDv0)', async () => {
    const data = await ipfs.files.cat(`/ipfs/${file.cid}`)
    expect(data).to.exist()
    const expectedContents = loadFixture('test/fixtures/testfile.txt')
    expect(data.toString()).to.equal(expectedContents.toString())
  })
})
